// ZipArchive.cpp: implementation of the ZipArchive class.
//
#include "StdAfx.h"
#include "ZipArchive.h"

#include "zlib.h"

#if defined(_AFXDLL)
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#endif

int __cdecl compareSearch(const void* elem1, const void* elem2)
{
	LPCTSTR pszString1 = (LPCTSTR)elem1;
	LPCTSTR pszString2 = ((ZENTRY*)*((DWORD*)elem2))->pFileName;
	return ::lstrcmpi(pszString1, pszString2);
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
ZipArchive::ZipArchive() :
	m_pStorage(NULL),
	m_pDirectory(NULL),
	m_bEntryOpened(FALSE)
{
}


ZipArchive::~ZipArchive()
{
}


BOOL ZipArchive::Open(IN LPCTSTR strFileSpec)
{
	BOOL bResult = FALSE;
	m_pStorage = (ZFIO*)zfOpen(strFileSpec);
	if (m_pStorage && zdOpenDirectory(&m_pDirectory, m_pStorage) == noError)
	{
		++bResult;
	}

	return bResult;
}

void ZipArchive::Close()
{
//	zdCloseDirectory(m_pDirectory);
//	m_pDirectory = NULL;

//	if (m_pStorage)
//		zfClose(m_pStorage);
}

LPCTSTR ZipArchive::LoadEntry(IN LONG idx, IN HGLOBAL& hBuffer)
{
	LPVOID pArray = &m_pDirectory->Array;
	ZENTRY* pEntry = (ZENTRY*)((DWORD*)pArray)[idx];
	return LoadEntry(pEntry, hBuffer);
}

LPCTSTR ZipArchive::LoadEntry(IN LPCTSTR lpszFileSpec, IN HGLOBAL& hBuffer)
{
	LPVOID pArray = &m_pDirectory->Array;
	ZENTRY* pEntry = (ZENTRY*)*(DWORD*)::BinarySearch((LPVOID)lpszFileSpec, pArray, m_pDirectory->Count, sizeof(ZENTRY*), compareSearch);
	return LoadEntry(pEntry, hBuffer);
}


LPCTSTR ZipArchive::LoadEntry(IN ZENTRY* pEntry, IN HGLOBAL& hBuffer)
{
	BOOL bResult = FALSE;
	hBuffer = NULL;

	if (pEntry)
	{
		DWORD dwFileSize;
		if (OpenEntry(pEntry, dwFileSize) == noError)
		{
			hBuffer = ::GlobalAlloc(GMEM_MOVEABLE, dwFileSize);
			if (hBuffer)
			{
				LPVOID pBuffer = (LPVOID)::GlobalLock(hBuffer);
				if (ReadEntry(pEntry, pBuffer, dwFileSize) == dwFileSize)
					bResult = TRUE;
				::GlobalUnlock(hBuffer);
			}
		}
		CloseEntry();
	}

	if (!bResult && hBuffer)
	{
		::GlobalFree(hBuffer);
		hBuffer = NULL;
	}

	return (bResult) ? pEntry->pFileName : NULL;
}

eZipError ZipArchive::OpenEntry(IN LPCTSTR strEntry, OUT DWORD& dwFileSize)
{
	LPVOID pArray = &m_pDirectory->Array;
	ZENTRY* pEntry = (ZENTRY*)*(DWORD*)::BinarySearch((LPVOID)strEntry, pArray, m_pDirectory->Count, sizeof(ZENTRY*), compareSearch);
	return OpenEntry(pEntry, dwFileSize);
}


eZipError ZipArchive::OpenEntry(IN ZENTRY* pEntry, OUT DWORD& dwFileSize)
{
	// if a zip entry is already opened or our ref ptr is NULL
	assert(!m_bEntryOpened);
	assert(pEntry != NULL);

	zfSeekBegin(m_pStorage, pEntry->pHeader->dwOffset);
	if (!zeReadLocal(pEntry, m_pStorage))
	{
		dwFileSize = 0;
		return badZipFile;
	}

	if ((pEntry->pHeader->Base.Flag & 1) != 0)
	{
		dwFileSize = 0;
		return badPassword;
	}

	WORD wMethod = pEntry->pHeader->Base.Method;
	if ((wMethod != Z_NO_COMPRESSION) && (wMethod != Z_DEFLATED))
	{
		dwFileSize = 0;
		return versionError;
	}
	
	m_uCrc32 = pEntry->pHeader->Base.Crc32;
	m_uComprLeft = pEntry->pHeader->Base.CompSize;
	m_uUncomprLeft = pEntry->pHeader->Base.UnCompSize;
	
	m_bEntryOpened = TRUE;
	dwFileSize = m_uUncomprLeft;
	return noError;
}

DWORD ZipArchive::ReadEntry(IN ZENTRY* pEntry, IN LPVOID pBuf, IN DWORD dwSize)
{
		// a zip file must be opened
	if (!m_bEntryOpened)
		return DWORD(-1);

	if (pEntry->pHeader->Base.Method == Z_NO_COMPRESSION)
	{
		LPVOID pBuffer = zfRead(m_pStorage, m_uUncomprLeft);
		CopyMemory(pBuf, pBuffer, m_uUncomprLeft);
		dwSize = m_uUncomprLeft;
	}
	else
	{
		if (m_uUncomprLeft != 0)
		{
			LPVOID pBuffer = zfRead(m_pStorage, m_uComprLeft);
			dwSize = inflate(pBuffer, m_uComprLeft, pBuf, m_uUncomprLeft);
			if (m_uCrc32 != Crc32(0, pBuf, dwSize))
				return 0;
		}
	}

	return dwSize;
}

void ZipArchive::CloseEntry()
{
	// a zip file must be opened
	if (m_bEntryOpened)
	{
		m_bEntryOpened = FALSE;
	}
}
