// ZipCentralDir.cpp: implementation of the ZipCentralDir class.
//
#include "StdAfx.h"
#include "ZipArchive.h"
#include "ZipCentralDir.h"

#if defined(_AFXDLL)
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#endif



const BYTE ZipCentralDir::m_cbSignature[] = {0x50, 0x4b, 0x05, 0x06};
const BYTE ZipCentralDir::m_cbExtHeaderSignature[] = {0x50, 0x4b, 0x07, 0x08};


int __cdecl compareSearch(const void* elem1, const void* elem2)
{
	LPCTSTR pszString1 = (LPCTSTR)elem1;
	LPCTSTR pszString2 = ((ZENTRY*)*((DWORD*)elem2))->pFileName;
	return ::lstrcmpi(pszString1, pszString2);
}

int __cdecl compareSort(const void* elem1, const void* elem2)
{
	LPCTSTR pszString1 = ((ZENTRY*)*((DWORD*)elem1))->pFileName;
	LPCTSTR pszString2 = ((ZENTRY*)*((DWORD*)elem2))->pFileName;
	return ::lstrcmpi(pszString1, pszString2);
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
ZipCentralDir::ZipCentralDir() :
	m_lArrayCount(0L),
	m_pArray(NULL)
{
}


int ZipCentralDir::OpenDirectory(IN LPVOID pArchive)
{
	// make sure we have a central directory record in this zip file
	int nRecSize = sizeof(CentralDirSignature);

	zfSeekEnd(pArchive, -nRecSize);
	lpCentralDirSignature pCentralDir = (lpCentralDirSignature)zfRead(pArchive, nRecSize);

	if (*((DWORD*)pCentralDir->Signature) != *((DWORD*)m_cbSignature))
		return noCentralDir;

	if ( (!pCentralDir->dwSize && pCentralDir->wNumEntries) ||
		  (!pCentralDir->wNumEntries && pCentralDir->dwSize))
	{
		return badZipFile;
	}

	if (pCentralDir->dwSize == 0 && pCentralDir->wNumEntries == 0)
		return zipDirEmpty;

	// go the the Central Directory offset in the file for reading
	// each of the file entries
	zfSeekBegin(pArchive, pCentralDir->dwOffset);

	// set our array size (or will soon be)
	m_lArrayCount = pCentralDir->wNumEntries;

	// create our array
	int nOffset = (m_lArrayCount << 2);
	int nAllocSize = nOffset + (sizeof(ZENTRY) * m_lArrayCount);
	m_pArray = (ZENTRY**) CreateMemory(nAllocSize, 0);

	// read each of the file entries
	for (int i = 0; i < m_lArrayCount; ++i)
	{
		// get the offset into the buffer to hold the ZENTRY info
		// and increment our offset
		ZENTRY* pEntry = (ZENTRY*)((LPBYTE)m_pArray + nOffset);
		nOffset += sizeof(ZENTRY);

		// save the offset into the buffer at the specified index value
		m_pArray[i] = pEntry;

		// read the ZENTRY information
		if (!zeRead(pEntry, pArchive))
			return badZipFile;
	}

	// sort the indexes so each one of our elements is in alpabetical order -- ascending
	::qsort(m_pArray, m_lArrayCount, sizeof(void*), compareSort);

	return noError;
}

// remove all headers from the central directory
void ZipCentralDir::CloseDirectory()
{
	if (m_lArrayCount)
	{
		DestroyMemory(m_pArray);
		m_pArray = NULL;
		m_lArrayCount = 0L;
	}
}
