// ---------------------------------------------------------------------------
// Zip.h: ordered includes for the Zip classes in the namespace of Zip
// ===========================================================================
#if !defined(AFX_ZIP_H__0081FC65_C9C9_4D48_AF72_DBF37DF5E0CF__INCLUDED_)
#define AFX_ZIP_H__0081FC65_C9C9_4D48_AF72_DBF37DF5E0CF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma pack(1)
typedef struct CentralDirectoryBaseHeader
{
   WORD	VerNeeded;		// version needed to extract
   WORD	Flag;				// general purpose bit flag
   WORD	Method;			// compression method
   WORD	ModTime;			// last modified time
   WORD	ModDate;			// last modified date
   DWORD	Crc32;			// CrC32
   DWORD	CompSize;		// compressed size
   DWORD	UnCompSize;		// uncompressed size
   WORD	FileNameSize;	// filename length
   WORD	ExtraField;		// extra field length
} typeCentralDirectoryBaseHeader, *lpCentralDirectoryBaseHeader;	// 26 bytes

typedef struct CentralDirectoryHeader
{
	BYTE		Signature[4];	// central file header signature (0x02014b50)
	WORD		VersionMadeBy;	// version of zip used to create
	CentralDirectoryBaseHeader Base;
	WORD		CommentSize;	// file comment info size
	WORD		DiskStart;		// disk start number
	WORD		InternalAttr;	// internal file attributes
	DWORD		ExternalAttr;	// external file attributes (attribues on disk)
	DWORD		dwOffset;		// relative offset of local header
} typeCentralDirectoryHeader, *lpCentralDirectoryHeader;	// 46 bytes

typedef struct CentralDirectoryLocalHeader
{
	BYTE	cbSignature[4];
	CentralDirectoryBaseHeader base;
} typeCentralDirectoryLocalHeader, *lpCentralDirectoryLocalHeader;	// 30 bytes

typedef struct CentralDirSignature
{
	BYTE		Signature[4];		// end of central dir signature (0x06054b50)
	WORD		wThisDisk;			// number of this disk
	WORD		wDiskWithCD;		// number of the disk with the start of the central directory
	WORD		wNumDiskEntries;	// total number of entries in the central directory on this disk
	WORD		wNumEntries;		// total number of entries in entire central directory
	DWORD		dwSize;				// size of the central directory
	DWORD		dwOffset;			// offset of start of central directory with respect to the starting disk number
	WORD		wCommentSize;		// zipfile comment length
} typeCentralDirSignature, *lpCentralDirSignature;	// 22 bytes
#pragma pack()


typedef struct ZFIO
{
	LPBYTE	pMemPosition;
	DWORD		dwFileSize;
	LPBYTE	pFileBuffer;
} typeZFIO, *lpZFIO;

typedef struct ZENTRY
{
	CentralDirectoryHeader* pHeader;
	LPTSTR	pFileName;
} typeZENTRY, *lpZENTRY;

typedef struct ZDIRECTORY
{
	long		Count;
	ZENTRY**	Array;
} typeZDIRECTORY, *lpZDIRECTORY;

typedef struct ZARCHIVE
{
	ZFIO*				pIO;
	ZDIRECTORY*		pDirectory;
} typeZARCHIVE, *lpZARCHIVE;


#define ENCR_HEADER_LEN 12

#define SLASH_WINDOWS	TCHAR('\\')
#define SLASH_UNIX		TCHAR('/')

// errors number definitions
#define ZIP_GENERIC_ERROR		(0x80000000)
#define ZIP_BADZIPFILE			(0x80000001)
#define ZIP_BADCRC				(0x80000010)
#define ZIP_NOCALLBACK			(0x80000011)
#define ZIP_ABORTED				(0x80000100)
#define ZIP_NONREMOVABLE		(0x80000101)
#define ZIP_TOOMANYVOLUMES		(0x80000110)
#define ZIP_TOOLONGFILENAME	(0x80000111)
#define ZIP_BADPASSWORD			(0x80001000)
#define ZIP_CDIR_NOTFOUND		(0x80001001)

typedef enum eAllocType
{
	BufferNone = 0,
	BufferHeap,
	BufferHGlobal,
	BufferVirtual
} enumAllocType;

//	archive open modes	
typedef enum eOpenFlags
{
	zipModeNone				= (0x00000000L),
	zipModeRead				= (0x80000000L)
} enumOpenFlags;

typedef enum eZipError
{
	noError,		// no error
	generic,		// unknown error
	streamEnd,			// zlib library errors
	needDict,			//
	errNo,				//
	streamError,		//
	dataError,			//
	memError,			//	
	bufError,			//
	versionError,		//
	noCentralDir,
	badZipFile,		// damaged or not a zip file
	badCrc,			// crc mismatched
	tooLongFileName,	// the file name of the file added to the archive is too long
	badPassword,	// incorrect password set for the file being decrypted
	cdirNotFound,		///< the central directory was not found in the archive
	dirWithSize,		// during testing: found the directory with the size greater than 0
	zipDirEmpty,		// empty zip file
} enumZipError;


#endif	// !defined(AFX_ZIP_H__0081FC65_C9C9_4D48_AF72_DBF37DF5E0CF__INCLUDED_)
