TITLE	ZipArchive.asm

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive


include \masm32\include\windows.inc
include \masm32\include\kernel32.inc

include zip.inc		; structures, defines and prototypes for the ZIP library


externdef stdcall BinarySearch		:proto stdcall :dword, :dword, :dword, :dword, :dword
externdef stdcall Crc32					:proto stdcall :dword, :dword, :dword 

externdef stdcall inflate				:proto :dword, :dword, :dword, :dword

.data?
	dwProcessHeapID	dword ?


.code
align 4
; ----------------------------------------------------------------------------
; prototype:
; CHAR* __cdecl compareSearch(void* pElem1, void* pElem2);
; ----------------------------------------------------------------------------
compareSearch proc c Elem1:dword, Elem2:dword
	mov	edx, [Elem2]
	mov	eax, [Elem1]
	
	mov	ecx, [edx]
	mov	ecx, [ecx+4]		

	invoke lstrcmpi, eax, ecx	
	ret
compareSearch endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; BOOL __stdcall zaOpenArchive(ZARCHIVE* Instance);
; NOTE:
;		The responsibility of freeing of the buffers created in zfOpen and
;		zdOpenDirectory will be handled by zaCloseArchive so we can avoid two
;		function calls (zfClose and zdCloseArchive) which would contain a single
;		invoke statement to HeapFree -- basically, this is my way of handling
;		a compile time INLINE (like C/C++ can do).
; ----------------------------------------------------------------------------
zaOpenArchive proc uses ebx Instance:dword, FileSpec:dword
	; get our ProcessHeap handle
	invoke GetProcessHeap
	mov	dwProcessHeapID, eax

	mov	ebx, [Instance]
	
	; try and open the zip file
	invoke zfOpen, [FileSpec]
	test	eax, eax
	je		short @Failed
	
	; save our opened file information in our instance object
	mov	[ebx].zaInstance.IO, eax
	
	; read in the central directory now
	lea	eax, [ebx].zaInstance.Directory
	invoke zdOpenDirectory, eax, [ebx].zaInstance.IO
	test	eax, eax			; zipNoError
	jne	short @Failed

	mov	eax, 1
	ret
	
@Failed:
	xor	eax, eax
	ret
zaOpenArchive endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; void __stdcall zaCloseArchive(ZARCHIVE* Instance);
; ----------------------------------------------------------------------------
zaCloseArchive proc uses ebx Instance:dword
	mov	ebx, [Instance]

	mov	eax, [ebx].zaInstance.Directory
	test	eax, eax
	je		short @F

	invoke HeapFree, dwProcessHeapID, 0, eax

@@:
	mov	eax, [ebx].zaInstance.IO
	test	eax, eax
	je		short @F

;	invoke HeapFree, [dwProcessHeapID], 0, eax
	invoke VirtualFree, eax, 0, MEM_RELEASE

@@:
	ret
zaCloseArchive endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; LPCTSTR __stdcall zaLoadEntryByIndex(ZARCHIVE* Instance, int idx, HGLOBAL* pBufHandle);
; ----------------------------------------------------------------------------
zaLoadEntryByIndex proc Instance:dword, index:dword, BufHandle:dword
	; get the ZENTRY object from the ZARCHIVE Instance using the index value
	mov	eax, [Instance]
	mov	edx, [index]
	mov	ecx, [eax].zaInstance.Directory
	lea	edx, [ecx + edx*4 + 4]

	; call the actual loading routine now since we have the ZENTRY object
	invoke zaLoadEntry, [edx], [eax].zaInstance.IO, [BufHandle];	
	ret
zaLoadEntryByIndex endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; LPCTSTR __stdcall zaLoadEntryByName(ZARCHIVE* Instance, LPCTSTR FileSpec, HGLOBAL* pBufHandle);
; ----------------------------------------------------------------------------
zaLoadEntryByName proc uses ebx Instance:dword, FileSpec:dword, BufHandle:dword
	; get the ZENTRY object from the ZARCHIVE Instance using the filespec name
	mov	ebx, [Instance]
	mov	ecx, [ebx].zaInstance.Directory
	lea	edx, [ecx + 4]
	
	invoke BinarySearch, [FileSpec], edx, [edx].zdInstance.ArrayCount, sizeof dword, offset flat:compareSearch
	invoke zaLoadEntry, eax, [ebx].zaInstance.IO, [BufHandle]
	 
	ret
zaLoadEntryByName endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; LPCTSTR __stdcall zaLoadEntry(ZENTRY* Instance, ZIO* Instance, HGLOBAL* pBufHandle);
; ----------------------------------------------------------------------------
zaLoadEntry proc uses ebx edi Instance:dword, IO:dword, BufHandle:dword
	LOCAL	FileSize:dword
	LOCAL Result:dword
	
	mov	ebx, [Instance]
	mov	edi, [BufHandle]
	
	; make sure we have a valid zeEntry object pointer
	test	ebx, ebx
	mov	dword ptr [edi], 0
	je		short @Failed

	; open up the ZENTRY file for decompressing, just sets up
	; memory pointers, etc.  There isn't a corresponding close
	; because nothing needs to be freed, etc.
	invoke zaOpenEntry, [Instance], [IO], addr FileSize
	test	eax, eax		; zipNoError
	jne	short @Failed

	; if we dont' have a size, this is an empty item -- most likely
	; a to be a folder name
	cmp	[FileSize], 0
	je		short @Success
	
	; allocate our decompression buffer in a HGLOBAL handle
	; this allows the most flexible way of using the memory.
	; main reason it is GMEM_MOVABLE instead of GMEM_PTR is
	; because we can take this handle and directly pass it
	; to a GDI+ object for drawing (in case the buffer
	; is an image).
	;
	; Side effects require to additional calls to use this
	; handle for normal use.  GlobalLock and GlobalUnlock.
	; But in my opinion, it is worth it to have it this way.
	invoke GlobalAlloc, GMEM_MOVEABLE, [FileSize]
	test	eax, eax		; make sure we have a buffer
	mov	[edi], eax
	je		short @Failed
	
	; successful allocation, now decompress the zip entry
	; into this buffer with the GlobalLock, GlobalUnlock
	; wrapper calls
	invoke GlobalLock, eax
	invoke zaReadEntry, [Instance], [IO], eax
	mov	[Result], eax
	invoke GlobalUnlock, [edi]

	; sure we read decompressed the whole amount correctly
	mov	eax, [Result]
	cmp	eax, [FileSize]
	je		short @Success
	
	; free our buffer and return NULL for the filename
	invoke GlobalFree, [edi]
	and	dword ptr [edi], 0
	
@Failed:
	xor	eax, eax
	ret	
	
@Success:
	; return our filename of this entry 
	mov	eax, [ebx].zeInstance.FileName
	ret
	
zaLoadEntry endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; int __stdcall zaOpenEntryByName(ZARCHIVE* Instance, LPCTSTR FileSPec, DWORD* pFileSize);
; ----------------------------------------------------------------------------
zaOpenEntryByName proc uses ebx Instance:dword, FileSpec:dword, FileSize:dword
	
	; get the ZENTRY object from the ZARCHIVE Instance using the filespec name
	mov	ebx, [Instance]
	mov	ecx, [ebx].zaInstance.Directory
	lea	eax, [ecx + 4]
	
	invoke BinarySearch, [FileSpec], eax, [eax].zdInstance.ArrayCount, sizeof dword, offset flat:compareSearch
	invoke zaOpenEntry, eax, [ebx].zaInstance.IO, [FileSize]
	 
	ret
zaOpenEntryByName endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; int __stdcall zaOpenEntry(ZENTRY* Instance, DWORD* dwFileSize);
; ----------------------------------------------------------------------------
zaOpenEntry proc uses ebx edi Instance:dword, IO:dword, FileSize:dword
	; seek to the correct file position for this entry
	mov	ecx, [Instance]
	mov	edi, [FileSize]
	mov	ebx, [ecx].zeInstance.CDHdr
	mov	eax, [ebx].CentralDirectoryHeader.dwOffset
	invoke zfSeekBegin, [IO], eax

	; read in the local information for this entry
	invoke zeReadLocal, [Instance], [IO]
	test	eax, eax
	jne	short @F

	mov	dword ptr [edi], 0
	mov	eax, ZipBadZipFile
	ret

@@:
	; if the zip entry has a password, abort -- password support is not in this version
	mov	eax, dword ptr [ebx].CentralDirectoryHeader.Base.Flag
	test	eax, 1
	je		short @F
	
	mov	dword ptr [edi], 0
	mov	eax, ZipBadPassword
	ret
	
@@: 
	; make sure we have a Method if deflation (0 = UNCOMPRESSED, 8 = Z_DEFLATED)
	movzx	eax, word ptr [ebx].CentralDirectoryHeader.Base.Method
	test	eax, eax
	je		short @F
	cmp	eax, 8;		Z_DEFLATED
	je		short @F
	
	mov	dword ptr [edi], 0
	mov	eax, ZipVersionError
	ret
	
@@:
	mov	ecx, [ebx].CentralDirectoryHeader.Base.UnCompSize
	mov	eax, zipNoError
	mov	dword ptr [edi], ecx
	ret
zaOpenEntry endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; DWORD __stdcall zaReadEntry(ZENTRY* pEntry, LPVOID pBuf, DWORD dwSize);
; ----------------------------------------------------------------------------
zaReadEntry proc uses ebx esi edi Instance:dword, IO:dword, Buffer:dword
	mov	ecx, [Instance]
	mov	ebx, [ecx].zeInstance.CDHdr
	mov	edi, [Buffer]
	mov	edx, [ebx].CentralDirectoryHeader.Base.UnCompSize

	; see if the entry is compressed or not.
	movzx	eax, word ptr [ebx].CentralDirectoryHeader.Base.Method
	test	eax, eax		; no compression?
	jne	short @UnCompressIt

	; ---------------------
	; non-compressed entry
	; ---------------------
	
	; get a pointer to the uncompressed file memory contents
	invoke zfRead, [IO], edx
	mov	esi, eax
	
	; just do a straight copy of the zip entry contents from our file memory
	; buffer to the passed in buffer
	mov	edx, [ebx].CentralDirectoryHeader.Base.UnCompSize
	mov	ecx, edx
	shr	ecx, 2
	rep	movsd
	mov	ecx, edx
	and	ecx, 3
	rep	movsb

	mov	eax, edx
	ret
	
@UnCompressIt:
	; -----------------
	; compressed entry
	; -----------------

	; get a pointer to the compressed file memory contents
	invoke zfRead, [IO], [ebx].CentralDirectoryHeader.Base.CompSize
	
	; inflate (decompress it)
	push	[ebx].CentralDirectoryHeader.Base.UnCompSize
	push	edi
	push	[ebx].CentralDirectoryHeader.Base.CompSize
	push	eax
	call	inflate
	
	; validate and make sure the CRC32 values are the same now.
	invoke Crc32, 0, [Buffer], eax
	
	; compare the calculated CRC32 with the entries crc32
	cmp	eax, [ebx].CentralDirectoryHeader.Base.Crc32
	jne	@F
	
	; success
	mov	eax, [ebx].CentralDirectoryHeader.Base.UnCompSize
	ret
	
@@:	; failure
	and	eax, 0
	ret
zaReadEntry endp

END
