TITLE	ZipCentral.asm

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive

include \masm32\include\windows.inc
include \masm32\include\kernel32.inc

include zip.inc		; structures, defines and prototypes for the ZIP library

externdef c qsort 			:proto c :dword, :dword, :dword, :dword

; NOTE:
;		The responsibility of freeing of the buffer created in zdOpenDirecotry
;		is handled by the caller (located in zaCloseArchive) to avoid a function
;		call into this module that contains a single invoke statment to HeapFree.

.code
align 4
; ----------------------------------------------------------------------------
; prototype:
; CHAR* __cdecl compareSort(void* pElem1, void* pElem2);
; ----------------------------------------------------------------------------
compareSort proc c Elem1:dword, Elem2:dword
	mov	eax, [Elem1]
	mov	edx, [Elem2]
	
	mov	ecx, [eax]
	mov	eax, [ecx+4]
	
	mov	ecx, [edx]
	mov	ecx, [ecx+4]		

	invoke lstrcmpi, eax, ecx	
	ret
compareSort endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; BOOL __stdcall zdOpenDirectory(zdInstance* Instance, zfInstance* IO);
; ----------------------------------------------------------------------------
zdOpenDirectory proc uses ebx esi edi Instance:dword, IO:dword
	LOCAL		Directory:dword
	
	; start at the Central Directory for reading.  last XXX bytes of file.
	invoke zfSeekEnd, [IO], -sizeof CentralDirSignature
	
	; read in the central directory now
	invoke zfRead, [IO], sizeof CentralDirSignature
	mov	[Directory], eax

	; make sure the signature we read in matches correctly for this type
	mov	ecx, dword ptr CentralDirectorySignature
	cmp 	dword ptr [eax].CentralDirSignature.Signature, ecx
	je		short @F
	
	mov	eax, ZipNoCentralDir
	ret
	
@@:
	mov	ecx, [eax].CentralDirSignature.dwSize
	movzx ebx, word ptr [eax].CentralDirSignature.wNumEntries

	.if (!ecx && ebx) || (!ebx && ecx)
		mov	eax, ZipBadZipFile
		ret
	.endif

	.if (!ecx && !ebx)
		mov	eax, ZipDirEmpty
		ret
	.endif
	
	; go the the Central Directory offset in the file for reading
	; each of the file entries
	mov	ecx, [Directory]
	invoke zfSeekBegin, [IO], [ecx].CentralDirSignature.dwOffset
	
	; set up the array size/count and allocate memory to hold all the
	; entries which will be sorted in alphabetical order based on
	; their filenames.  the first section of the buffer will be
	; indexes into the entries which are in the last part of the
	; buffer allocated
	; array size = (NumEntries * sizeof(zeInstance)) + (NumEntries * sizeof(dword)) + sizeof(dword)
	mov   ecx, ebx 
   mov   eax, sizeof zeInstance 
   imul  eax, ebx      ; NumEntries * sizeof(zeInstance) 
	mov   esi, ebx 
   lea	ecx, [ecx*4+eax+4] 
   invoke HeapAlloc, [dwProcessHeapID], 0, ecx 

	; save our allocation and array count
   mov   ecx, [Instance] 
   mov   [eax].zdInstance.ArrayCount, ebx 
   mov   [ecx], eax 

	; set up registers for looping to process each of the array elements
	; ebx = counter
	; esi = index into array to store Element (stores edi)
	; edi = address of element array
   lea  	edi, [eax+4+ebx*4] 
   lea  	esi, [eax+4] 

	; now read each of the file entries into the buffer -- this is just
	; their directory record information and not the contents of the file
@Loop: 
   mov   [esi], edi 
   sub   ebx, 1 
   add   esi, 4 
   invoke zeRead, edi, [IO] 
   add   edi, sizeof zeInstance 
   test  eax, eax 
   je    short @Failed 
   test  ebx, ebx 
   jne   short @Loop 

	; sort our array indexes using qsort with the filename of each entry
	; in ascending order.
;	mov	edx, [Instance]
;	mov	edx, [edx]
;	mov	eax, [edx].zdInstance.ArrayCount
;	add	edx, sizeof dword
;	invoke qsort, edx, eax, sizeof dword, offset flat:compareSort
;	add	esp, 16

	; success
	mov	eax, zipNoError
	ret
	
@Failed:
	mov	eax, ZipBadZipFile
	ret
zdOpenDirectory endp


END
