TITLE	zip.inc

; ----------------------------------------------------------------------------
; structures
; ----------------------------------------------------------------------------

.code
align 1		; must be aligned by 'one' to read from disk.
CentralDirectoryBaseHeader struct
   VerNeeded		word ?    ; version needed to extract
   Flag				word ?    ; general purpose bit flag
   Method			word ?    ; compression method
   ModTime			word ?    ; last modified time
   ModDate			word ?    ; last modified date
   Crc32				dword ?   ; CrC32
   CompSize			dword ?   ; compressed size
   UnCompSize		dword ?   ; uncompressed size
   FileNameSize	word ?    ; filename length
   ExtraField		word ?    ; extra field length
CentralDirectoryBaseHeader ends


align 1		; must be aligned by 'one' to read from disk.
CentralDirectoryHeader struct
	Signature		byte 4 dup (?)		; central file header signature (0x02014b50)
	VersionMadeBy	word ?				; version of zip used to create
	Base				CentralDirectoryBaseHeader<>
	CommentSize		word ?				; file comment info size
	DiskStart		word ?				; disk start number
	InternalAttr	word ?				; internal file attributes
	ExternalAttr	dword ?				; external file attributes (attribues on disk)
	dwOffset			dword ?				; relative offset of local header
CentralDirectoryHeader ends


align 1		; must be aligned by 'one' to read from disk.
CentralDirectoryLocalHeader struct
	Signature		byte 4 dup (?)
	Base				CentralDirectoryBaseHeader<>
CentralDirectoryLocalHeader ends


align 1		; must be aligned by 'one' to read from disk.
CentralDirSignature struct
	Signature			byte 4 dup (?)		; end of central dir signature (0x06054b50)
	wThisDisk			word ?				; number of this disk
	wDiskWithCD			word ?				; number of the disk with the start of the central directory
	wNumDiskEntries	word ?				; total number of entries in the central directory on this disk
	wNumEntries			word ?				; total number of entries in entire central directory
	dwSize				dword ?				; size of the central directory
	dwOffset				dword ?				; offset of start of central directory with respect to the starting disk number
	wCommentSize		word ?				; zipfile comment length
CentralDirSignature ends


align 16
; ----------------------------------------------------------------------------
; instances of objects
; ----------------------------------------------------------------------------
zfInstance struct
	MemPosition		dword ?		; current position in the MemBuffer
	MemBufSize		dword ?		; size of MemBuffer
	MemBuffer		dword ?		; file contents 
zfInstance ends

zeInstance struct
	CDHdr				dword ?		; pointer to central directory header record
	FileName			dword ?		; pointer to a null-terminated 'filename'
zeInstance ends

zdInstance struct
	ArrayCount		dword ?		; number of elements in array
	Array				dword ?		; array of ZipEntry elements
zdInstance ends

zaInstance struct
	IO					dword ?		; pointer to a ZipFile instance
	Directory		dword ?		; pointer to a ZipDirectory instance 
zaInstance ends


align 16
; ----------------------------------------------------------------------------
; memory structures
; ----------------------------------------------------------------------------
code	struct
	op			byte ?	; operation, extra bits, table bits
	bits		byte ?	; bits in this part of the code
	val		word ?	; offset in table or code value
code ends


; state information for inflate, inflate_table and inflate_fast
InflateState struct
	; bit accumulator variables
	hold			dword ?	; input bit accumulator
	bits			dword ?	; number of bits in "in" working buffer
	
	; for string and stored block copying
	nLength		dword ? 	; literal or length of data to copy
	nOffset		dword ?	; distance back to copy string from
	
	; for table and code decoding
	extra			dword ?	; extra bits needed
	
	; fixed and dynamic code tables
	lenbits		dword ?	; index bits for lencode
	distbits		dword ? 	; index bits for distcode
	
	; dynamic table building
	ncode			dword ?	; number of code length code lengths
	nlen			dword ?	; number of length code lengths
	ndist			dword ?	; number of distance code lengths
	have			dword ?	; number of code lengths in lens[]
	
	; code* variables
	next			dword ?	; next available space in codes[]
	lencode		dword ?	; starting table for length/literal codes
	distcode		dword ?	; starting table for distance codes
InflateState ends

; basic Stream information
ZipStream struct
	next_in		dword ? 	; next input byte
	next_out		dword ?	; next output byte should be put here
	
	avail_in		dword ? 	; number of bytes available at next_in
	avail_out	dword ?	; remaining free space at next_out
	
	total_in		dword ?	; total number of input bytes read so far
	total_out	dword ?	; totoal nubmer of bytes output so far
	
	State			InflateState<>
ZipStream ends


align 16
.const
; allowed flushed values
Z_NO_FLUSH			equ 0
Z_SYNC_FLUSH		equ 2
Z_PARTIAL_FLUSH	equ Z_SYNC_FLUSH
Z_FULL_FLUSH		equ 3
Z_FINISH				equ 4
Z_BLOCK				equ 5


; return codes for the compress / decompression functions.
; Negative values are errors, positive values are used for
; special but normal events.
; example:  Decompress() returns Z_STREAM_END when completed
Z_OK					equ 0
Z_STREAM_END		equ 1
Z_NEED_DICT			equ 2
Z_ERRNO				equ -1
Z_STREAM_ERROR		equ -2
Z_DATA_ERROR		equ -3
Z_MEM_ERROR			equ -4
Z_BUF_ERROR			equ -5
Z_VERSION_ERROR	equ -6


; compression levels
Z_NO_COMPRESSION			equ 0
Z_BEST_SPEED				equ 1
Z_BEST_COMPRESSION		equ 9
Z_DEFAULT_COMPRESSION	equ -1


; compression strategy
Z_DEFAULT_STRATEGY	equ 0
Z_FILTERED				equ 1
Z_HUFFMAN_ONLY			equ 2
Z_RLE						equ 3


; possible values of the data-type field
Z_BINARY		equ 0
Z_ASCII		equ 1
Z_UNKNOWN	equ 2


; The deflate compression method (the only one supported in this version)
Z_DEFLATED		equ 8


; Decompress switch table mode values
Z_TABLE_HEAD			equ 0
Z_TABLE_FLAGS			equ 1
Z_TABLE_TIME			equ 2
Z_TABLE_OS				equ 3
Z_TABLE_EXLEN			equ 4
Z_TABLE_EXTRA			equ 5
Z_TABLE_NAME			equ 6
Z_TABLE_COMMENT		equ 7
Z_TABLE_HCRC			equ 8
Z_TABLE_DICTID			equ 9
Z_TABLE_DICT			equ 0ah
Z_TABLE_TYPE			equ 0bh
Z_TABLE_TYPDO			equ 0ch
Z_TABLE_STORED			equ 0dh
Z_TABLE_COPY			equ 0eh
Z_TABLE_TABLE			equ 0fh
Z_TABLE_LENSLENS		equ 10h
Z_TABLE_CODELENS		equ 11h
Z_TABLE_LEN				equ 12h
Z_TABLE_LENEXT			equ 13h
Z_TABLE_DIST			equ 14h
Z_TABLE_DISTEXT		equ 15h
Z_TABLE_MATCH			equ 16h
Z_TABLE_LIT				equ 17h
Z_TABLE_CHECK			equ 18h
Z_TABLE_LENGTH			equ 19h
Z_TABLE_DONE			equ 1ah
Z_TABLE_BAD				equ 1bh
Z_TABLE_MEM				equ 1ch
Z_TABLE_SYNC			equ 1dh


comment #
   Structure for decoding tables.  Each entry provides either the
   information needed to do the operation requested by the code that
   indexed that table entry, or it provides a pointer to another
   table that indexes more bits of the code.  op indicates whether
   the entry is a pointer to another table, a literal, a length or
   distance, an end-of-block, or an invalid code.  For a table
   pointer, the low four bits of op is the number of index bits of
   that table.  For a length or distance, the low four bits of op
   is the number of extra bits to get after the code.  bits is
   the number of bits in this code or part of the code to drop off
   of the bit buffer.  val is the actual byte to output in the case
   of a literal, the base length or distance, or the offset from
   the current table to the next table.  Each entry is four bytes.


   op values as set by inflate_table():
    00000000 - literal
    0000tttt - table link, tttt != 0 is the number of table index bits
    0001eeee - length or distance, eeee is the number of extra bits
    01100000 - end of block
    01000000 - invalid code
#


; Maximum size of dynamic tree.  The maximum found in a long but non-
; exhaustive search was 1004 code structures (850 for length/literals
; and 154 for distances, the latter actually the result of an
; exhaustive search).  The true maximum is not known, but the value
; below is more than safe.
Z_ENOUGH		equ  05a0h
Z_MAXD		equ  09ah


; type of code to build for inflate_table
CODE_TABLE_CODES		equ 0
CODE_TABLE_LENS		equ 1
CODE_TABLE_DISTS		equ 2


; ----------------------------------------------------------------------------
; constants
; ----------------------------------------------------------------------------
.const
align 16
EntrySignature						db 050h, 04bh, 01h, 02h
EntryLocalSignature				db	050h, 04bh, 03h, 04h
CentralDirectorySignature		db 050h, 04bh, 05h, 06h
CentralDirectoryExtSignature	db 050h, 04bh, 07h, 08h


; ----------------------------------------------------------------------------
; equates
; ----------------------------------------------------------------------------
align	16
.const
SLASH_WINDOWS			equ 05cH		; '\\'
SLASH_UNIX				equ 02fH		; '/'


zipNoError				equ 0			; no error
ZipGeneric				equ 1			; unknown error
ZipStreamEnd			equ 2			; zlib library errors
ZipNeedDict				equ 3			;
ZipErrNumber			equ 4			;
ZipStreamError			equ 5			;
ZipDataError			equ 6			;
ZipMemError				equ 7			;	
ZipBufError				equ 8			;
ZipVersionError		equ 9			;
ZipNoCentralDir		equ 0ah		;
ZipBadZipFile			equ 0bh		; damaged or not a zip file
ZipBadCrc				equ 0ch		; crc mismatched
ZipTooLongFileName	equ 0dh		; added file name is too long
ZipBadPassword			equ 0eh		; incorrect password set for the file being decrypted
ZipCdirNotFound		equ 0fh		; the central directory was not found in the archive
ZipDirWithSize			equ 10h		; during testing: found the directory with the size greater than 0
ZipDirEmpty				equ 11h		; empty zip file


; ----------------------------------------------------------------------------
; externed function call identifiers
; ----------------------------------------------------------------------------
; search and compare helper functions
externdef c compareSearch					:proto c :dword, :dword
externdef c compareSort						:proto c :dword, :dword

; file IO calls --- private calls for the libarary
externdef stdcall zfOpen					:proto stdcall :dword
externdef stdcall zfRead					:proto stdcall :dword, :dword
externdef stdcall zfSeekBegin				:proto stdcall :dword, :dword
externdef stdcall zfSeekCurrent			:proto stdcall :dword, :dword
externdef stdcall zfSeekEnd				:proto stdcall :dword, :dword

; Individual items in a zip file calls --- private calls for the libarary
externdef stdcall zeRead					:proto stdcall :dword, :dword
externdef stdcall zeReadLocal				:proto stdcall :dword, :dword

; Central directory of a zip file --- private calls for the libarary
externdef stdcall zdOpenDirectory		:proto stdcall :dword, :dword

; Main calls to access a zip file, its contents, etc. -- these are all
; public calls while the rest of the calls (zf???, ze???, zd???) are
; private to the library.
externdef stdcall zaOpenArchive			:proto stdcall :dword, :dword
externdef stdcall zaCloseArchive			:proto stdcall :dword
externdef stdcall zaGetEntryCount		:proto stdcall :dword
externdef stdcall zaGetEntryArray		:proto stdcall :dword
externdef stdcall zaLoadEntryByName		:proto stdcall :dword, :dword, :dword
externdef stdcall zaLoadEntryByIndex	:proto stdcall :dword, :dword, :dword
externdef stdcall zaLoadEntry				:proto stdcall :dword, :dword, :dword 
externdef stdcall zaOpenEntry				:proto stdcall :dword, :dword, :dword
externdef stdcall zaOpenEntryByName		:proto stdcall :dword, :dword, :dword
externdef stdcall zaReadEntry				:proto stdcall :dword, :dword, :dword


.data?
align 16
; ----------------------------------------------------------------------------
; externed variables used by multiple modules
; ----------------------------------------------------------------------------
externdef	dwProcessHeapID :dword
