TITLE	zipEntry.asm

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive


include \masm32\include\windows.inc
include \masm32\include\kernel32.inc
include \masm32\include\shlwapi.inc

include zip.inc		; structures, defines and prototypes for the ZIP library


.code
align 4
; ----------------------------------------------------------------------------
; prototype:
; static CHAR* __stdcall zeAddNullToZipString(const char* pszFileSpec, const int nLen);
; ----------------------------------------------------------------------------
zeAddNullToZipString proc uses ebx esi edi String:dword, Len:dword
	mov	eax, [Len]
	mov	esi, [String]
	mov	ebx, eax
	add	eax, 1

	invoke HeapAlloc, [dwProcessHeapID], HEAP_ZERO_MEMORY, eax	
	
	mov	ecx, ebx
	mov	edi, eax
	
	shr	ecx, 2
	and	ebx, 3
	rep	movsd
	mov	ecx, ebx
	rep	movsb
	ret
zeAddNullToZipString endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; static void __stdcall zeSlashChange(ZipEntry* pEntry, CHAR cFrom, CHAR cTo);
; ----------------------------------------------------------------------------
zeSlashChange proc uses ebx esi edi Instance:dword, From:byte, To:byte
	mov	ecx, [Instance]
	mov	esi, [ecx].zeInstance.FileName

	mov	bl, byte ptr [To]
	movzx edi, byte ptr [From]
	
@Loop:
	invoke StrChr, esi, edi		; SHLWAPI.DLL  call
	test	eax, eax
	je		short @Done

	mov	byte ptr [eax], bl
	mov	esi, eax
	jmp	short @Loop
	
@Done:	
	ret
zeSlashChange endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; BOOL __stdcall zeRead(ZipEntry* pEntry, StorageIO* pStorage);
; ----------------------------------------------------------------------------
zeRead proc uses ebx esi edi Instance:dword, IO:dword
	mov	edi, [Instance]
	mov	ebx, offset flat:EntrySignature

	; read in the header from the zip file
	invoke zfRead, [IO], sizeof CentralDirectoryHeader
	mov	ecx, [ebx]
	mov	[edi].zeInstance.CDHdr, eax

	; validate our signature with that from the Central Directory
	cmp	ecx, dword ptr [eax].CentralDirectoryHeader.Signature
	jne	short @Failed
	
	; see if we have a filename
	mov	esi, [edi].zeInstance.CDHdr
	movzx	ebx, word ptr [eax].CentralDirectoryHeader.Base.FileNameSize 
	test	ebx, ebx
	je		short @F
	
	; get a pointer to the filename
	invoke zfRead, [IO], ebx
	
	; convert the filename from a non-null terminated to a null-terminated string
	invoke zeAddNullToZipString, eax, ebx
	mov	[edi].zeInstance.FileName, eax

	; get ready to check the Extra Field and Comment to see if we are skipping past
	movzx	eax, word ptr [esi].CentralDirectoryHeader.Base.ExtraField
	movzx	ebx, word ptr [esi].CentralDirectoryHeader.CommentSize
	
@@:
	; skip passed any extra field information
	test	eax, eax
	je		short @F
	invoke zfSeekCurrent, [IO], eax
	
@@:
	; skip passed any comment information
	test	ebx, ebx 
	je		short @F
	invoke zfSeekCurrent, [IO], ebx

@@:
	; convert the slashes to windows style.
	invoke zeSlashChange, [Instance], SLASH_UNIX, SLASH_WINDOWS
	
	; success
	mov	eax, 1
	ret
		
@Failed:
	xor	eax, eax
	ret
zeRead endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; BOOL __stdcall zeReadLocal(ZipEntry* pEntry, StorageIO* pStorage);
; ----------------------------------------------------------------------------
zeReadLocal proc uses ebx esi edi Instance:dword, IO:dword
	; read in the local header information for this zip entry
	invoke zfRead, [IO], sizeof CentralDirectoryLocalHeader

	; validate the local signature with the directory signature
	mov	ecx, offset flat:EntryLocalSignature
	mov	ebx, [Instance]

	mov	ecx, [ecx]
	mov	ebx, [ebx].zeInstance.CDHdr

	cmp	[eax], ecx
	jne	short @Failed

	; validate basic information between entry and central directory.
	; these values must be the same
	lea	esi, dword ptr [eax].CentralDirectoryLocalHeader.Base
	lea	edi, dword ptr [ebx].CentralDirectoryHeader.Base
	
	; sizeof(CentralDirectoryBaseHeader) - sizeof(word) should always be divisible by four
	mov	ecx, (sizeof CentralDirectoryBaseHeader - sizeof word) / 4
	repe	cmpsd
	je		short @F
	
	; since they aren't equal, we now need to check and see if this
	; entry is a data descriptor.
	test	byte ptr [eax].CentralDirectoryLocalHeader.Base.Flag, 8
	jne	short @F
	
	cmp	[eax].CentralDirectoryLocalHeader.Base.Crc32, 0
	jne	short @Failed

@@:
	; I tried putting these directly on the invoke statements, but the sizes
	; were wrong.  :(   So, these two extra moves fixes my problem -- maybe someone
	; can see what I was missing.
	movzx	esi, word ptr [eax].CentralDirectoryLocalHeader.Base.FileNameSize
	movzx	edi, word ptr [eax].CentralDirectoryLocalHeader.Base.ExtraField
	
	; seek past our local filename
	invoke zfSeekCurrent, [IO], esi
	
	; seek past extra field data
	invoke zfSeekCurrent, [IO], edi

@Done:
	; success
	mov	eax, 1
	ret
	
@Failed:
	xor	eax, eax	
	ret
zeReadLocal endp

END
