TITLE	zipFileIO.asm

.586
.model flat, stdcall      ; memory model & calling convention
option casemap :none      ; case sensitive

include \masm32\include\windows.inc
include \masm32\include\kernel32.inc

include zip.inc		; structures, defines and prototypes for the ZIP library


; NOTE:
;		The responsibility of freeing of the buffer created in zfOpen is handled
;		by the caller (located in zaCloseArchive) to avoid a function call into
;		this module that contains a single invoke statment	to HeapFree.

.code
align 4
; ----------------------------------------------------------------------------
; prototype:
; void* __stdcall zfOpen(const char* pszFileSpec);
; ----------------------------------------------------------------------------
zfOpen proc uses ebx esi edi FileSpec:dword
	LOCAL ReadAmount:dword
	
	; open the zip file specified
	invoke CreateFile, [FileSpec], GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL or FILE_FLAG_SEQUENTIAL_SCAN, NULL
	
	; make sure the file was successfully opened
	test	eax, eax
	je		short @F
	
	; save our file handle
	mov	edi, eax
	
	; get the size of the file
	invoke GetFileSize, eax, NULL
	mov	ebx, eax
	
	; create the instance data plus room to read the contents of the file into
	add	eax, 8
	
	invoke VirtualAlloc, NULL, eax, MEM_COMMIT, PAGE_READWRITE
;	invoke HeapAlloc, [dwProcessHeapID], 0, esi
	mov	esi, eax
	
	; start our memory pointer offset at the beginning of the buffer
	add	eax, 8
	mov	[esi].zfInstance.MemPosition, eax

	; save our filesize
	mov	[esi].zfInstance.MemBufSize, ebx
	
	; read in the file contents
	lea	eax, [ReadAmount]
	lea	ecx, [esi].zfInstance.MemBuffer
	invoke ReadFile, edi, ecx, ebx, eax, NULL
	
	; close our file handle
	invoke CloseHandle, edi
	
	; move the instance stuff into eax for returning
	mov	eax, esi
		
@@:
	ret	
zfOpen endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; void* __stdcall zfRead(void** Instance, DWORD dwAmount);
; ----------------------------------------------------------------------------
zfRead proc Instance:dword, Amount:dword
	; return position of current memory pointer
	mov	ecx, [Instance]
	mov	edx, [Amount]
	mov	eax, [ecx].zfInstance.MemPosition
		
	; move the current memory pointer to the new position
	add	[ecx].zfInstance.MemPosition, edx
	ret
zfRead endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; void __stdcall zfSeekBegin(void** Instance, int offset);
; ----------------------------------------------------------------------------
zfSeekBegin proc Instance:dword, nOffset:dword
	; start the memory position pointer at the beginning of the file buffer
	mov	ecx, [Instance]
	lea	eax, [ecx].zfInstance.MemBuffer
	
	; add the offset to our memory position pointer
	add	eax, [nOffset]		; positive value
	
	; save the current memory position pointer into the file
	mov	[ecx].zfInstance.MemPosition, eax
	ret
zfSeekBegin endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; void __stdcall zfSeekCurrent(void** Instance, int offset);
; ----------------------------------------------------------------------------
zfSeekCurrent proc Instance:dword, nOffset:dword
	; begin at the current memory position pointer
	mov	ecx, [Instance]
	mov	eax, [ecx].zfInstance.MemPosition
	
	; add the offset to our memory position pointer
	add	eax, [nOffset]		; positive or negative value
	
	; save the current memory position pointer into the file
	mov	[ecx].zfInstance.MemPosition, eax
	ret
zfSeekCurrent endp


align 4
; ----------------------------------------------------------------------------
; prototype:
; void __stdcall zfSeekEnd(void** Instance, int offset);
; ----------------------------------------------------------------------------
zfSeekEnd proc Instance:dword, nOffset:dword
	; start the memory position pointer at the end of the file buffer
	mov	ecx, [Instance]
	lea	eax, [ecx].zfInstance.MemBuffer
		
	; add the offset to our memory position pointer
	add	eax, [ecx].zfInstance.MemBufSize
	add	eax, [nOffset]		; negative value
	
	; save the current memory position pointer into the file
	mov	[ecx].zfInstance.MemPosition, eax
	ret
zfSeekEnd endp

END
